from django.shortcuts import render
from django.views.generic import ListView
from queryset import RestFrameworkQuerySet


class ListDataView(ListView):
    paginate_by = 10
    template_name = 'list.html'

    def get_queryset(self, *args, **kwargs):
        return RestFrameworkQuerySet('http://localhost:8082/api/').filter(**self.request.GET.dict())



from django.test import LiveServerTestCase
from api.models import DataModel


class ListTestCase(LiveServerTestCase):
    def setUp(self):
        # create data
        for i in range(100):
            DataModel.objects.create(value=i)

    def test_list(self):
        resp = self.client.get('/')
        self.assertEqual([v['value'] for v in resp.context['object_list']], [i for i in range(10)])
        resp = self.client.get('/page/2/')
        self.assertEqual([v['value'] for v in resp.context['object_list']], [i for i in range(10, 20)])

    def test_filter(self):
        resp = self.client.get('/?value=10')
        self.assertEqual(len(resp.context['object_list']), 1)
        self.assertEqual(resp.context['object_list'][0]['value'], 10)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import requests
import copy


class BaseAPIQuerySet(object):
    def __init__(self, url, *args, **kwargs):
        self.request_method = requests.get
        self.url = url
        self.args = args
        self.kwargs = kwargs

    def call_api(self):
        """
        perform api call
        """
        return self.request_method(self.url, *self.args, **self.kwargs)

    def __call__(self):
        resp = self.call_api()
        return self.get_result(resp)

    def __len__(self):
        return self.get_count()

    def __getitem__(self, index):
        if isinstance(index, int):
            return super(APIQuerySet, self).__getitem__(index)
        elif isinstance(index, slice):
            return self.page_result(index)

    def __enter__(self):
        # temp swap out kwargs, assign by value and keep the original ref
        self._old_kwargs = self.kwargs
        self.kwargs = copy.deepcopy(self.kwargs)
        self._old_args = self.args
        self.args = copy.deepcopy(self.args)

    def __exit__(self, type, value, traceback):
        # put origin kwargs back
        self.kwargs = self._old_kwargs
        self.args = self._old_args

    def filter(self, **kargs):
        raise NotImplementedError()

    def get_count(self):
        raise NotImplementedError()

    def get_result(self):
        raise NotImplementedError()

    def page_result(self):
        raise NotImplementedError()


class RestFrameworkQuerySet(BaseAPIQuerySet):
    def get_count(self):
        with self:
            params = self.kwargs.get('params', {})
            params['page_size'] = 1
            self.kwargs['params'] = params
            resp = self.call_api()
            result = resp.json()
            return result['count']

    def get_result(self, response):
        result = response.json()
        return result['results']

    def page_result(self, slicer):
        with self:
            params = self.kwargs.get('params', {})
            params['offset'] = slicer.start
            params['limit'] = slicer.stop - slicer.start
            self.kwargs['params'] = params
            return self.__call__()

    def filter(self, **kwargs):
        params = self.kwargs.setdefault('params', {})
        params.update(kwargs)
        return self



from rest_framework import generics
from rest_framework import serializers
from .models import DataModel
from .filters import DataModelFilter


class DataModelSerializer(serializers.ModelSerializer):
    class Meta(object):
        model = DataModel


class ListView(generics.ListAPIView):
    serializer_class = DataModelSerializer
    queryset = DataModel.objects.all()
    filter_class = DataModelFilter



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from rest_framework.pagination import BasePagination, PageNumberPagination, LimitOffsetPagination


class HybridPagination(BasePagination):
    proxy = PageNumberPagination()
    limit_offset_pagination = LimitOffsetPagination()

    def paginate_queryset(self, queryset, request, view=None):
        if 'offset' in request.GET:
            self.proxy = self.limit_offset_pagination
        return self.proxy.paginate_queryset(queryset, request, view)

    def get_paginated_response(self, data):
        return self.proxy.get_paginated_response(data)

    def to_html(self):
        return self.proxy.to_html()

    def get_results(self, data):
        return self.proxy.get_results(data)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import codecs
import os
import re

from pip.download import PipSession
from pip.req import parse_requirements
from setuptools import setup, find_packages


def read(*parts):
    filename = os.path.join(os.path.dirname(__file__), *parts)
    with codecs.open(filename, encoding='utf-8') as fp:
        return fp.read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


def requirements(file_name='requirements.txt'):
    return [
        str(ir.req)
        for ir in parse_requirements(file_name, session=PipSession())
        if ir.match_markers()
    ]


def extra_requirements():
    try:
        join = os.path.join
        path = join('requirements', 'extra')
        return {
            str(file_name.rsplit('.', 1)[0]): requirements(join(path, file_name))
            for file_name in os.listdir(path)
            if file_name.endswith('.txt')
        }
    except OSError:
        return None

setup(
    name='django-rest-framework-queryset',
    version=find_version('__init__.py'),
    author='James Lin',
    author_email='james@lin.net.nz',
    long_description=read('README.md'),
    install_requires=requirements(),
    packages=find_packages(exclude=["tests"]),
    include_package_data=True,
    license='MIT',
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.5",
        "Framework :: Django",
    ],
    extras_require=extra_requirements(),
)
